class ApiError(Exception):
    '''General error from the Algorithmia API'''
    pass

class ApiInternalError(ApiError):
    '''Error representing a server error, typically a 5xx status code'''
    pass

class DataApiError(ApiError):
    '''Error returned from the Algorithmia data API'''
    pass

class ModelApiError(ApiError):
    '''Error returned from the Algorithmia model API'''
    pass




class AsyncResponse(object):
    '''
    Response from the API for an asynchronous request (output=void)
    '''
    def __init__(self, server_response):
        self.async_protocol = server_response['async']
        self.request_id = server_response['request_id']

    def __repr__(self):
        return 'AsyncResponse(async_protocol=%s, request_id=%s)' % (self.async_protocol, self.request_id) 


import base64

class AlgmResponse(object):
    def __init__(self, result, metadata):
        self.result = result
        self.metadata = metadata

    def __repr__(self):
        return 'AlgmResponse(result=%s,metadata=%s)' % (self.result, self.metadata)

    @staticmethod
    def create_algo_response(responseJson):
        # Parse response JSON
        if 'error' in responseJson:
            # Failure
            raise AlgmException(responseJson['error'])
        else:
            metadata = Metadata(responseJson)
            # Success, check content_type
            # if responseJson['metadata']['content_type'] == 'binary':
            #     # Decode Base64 encoded binary file
            #     return AlgmResponse(base64.b64decode(responseJson['result']), metadata)
            # elif responseJson['metadata']['content_type'] == 'void':
            #     return AlgmResponse(None, metadata)
            # else:
            return AlgmResponse(responseJson['data'], metadata)


class AlgmException(Exception):
    def __init__(self, error):
        self.message = error['message']
        self.stacktrace = None
        if 'stacktrace' in error:
            self.stacktrace = error['stacktrace']

    def __str__(self):
        return self.message

class Metadata(object):
    def __init__(self, metadata):
        self.duration = metadata['duration']
        self.stdout = None
        if 'stdout' in metadata:
            self.stdout = metadata['stdout']
        self.full_metadata = metadata

    def __repr__(self):
        return "Metadata(duration=%s,stdout=%s)" % ( self.duration, self.stdout)



'Algorithmia Data API Client (python)'

import json
import re
import six
import tempfile

import Algmarket
from Algmarket.datafile import DataFile
from Algmarket.data import DataObject, DataObjectType
from Algmarket.errors import DataApiError
from Algmarket.util import getParentAndBase, pathJoin
from Algmarket.acl import Acl

class DataDirectory(DataObject):
    def __init__(self, client, dataUrl):
        super(DataDirectory, self).__init__(DataObjectType.directory)
        self.client = client
        # Parse dataUrl
        self.path = re.sub(r'^data://|^/', '', dataUrl)
        self.url = DataDirectory._getUrl(self.path)

    @staticmethod
    def _getUrl(path):
        return '/v1/data/' + path

    def set_attributes(self, response_json):
        # Nothing to set for now
        pass

    def getName(self):
        _, name = getParentAndBase(self.path)
        return name

    def exists(self):
        # Heading a directory apparently isn't a valid operation
        response = self.client.getHelper(self.url)
        return (response.status_code == 200)

    def create(self, acl=None):
        '''Creates a directory, optionally include Acl argument to set permissions'''
        parent, name = getParentAndBase(self.path)
        json = { 'name': name }
        if acl is not None:
            json['acl'] = acl.to_api_param()
        response = self.client.postJsonHelper(DataDirectory._getUrl(parent), json, False)
        if (response.status_code != 200):
            raise DataApiError("Directory creation failed: " + str(response.content))

    def delete(self, force=False):
        # Delete from data api
        url = self.url
        if force:
            url += '?force=true'

        result = self.client.deleteHelper(url)
        if 'error' in result:
            raise DataApiError(result['error']['message'])
        else:
            return True

    def file(self, name):
        return DataFile(self.client, pathJoin(self.path, name))

    def files(self):
        return self._get_directory_iterator(DataObjectType.file)

    def dir(self, name):
        return DataDirectory(self.client, pathJoin(self.path, name))

    def dirs(self):
        return self._get_directory_iterator(DataObjectType.directory)

    def list(self):
        return self._get_directory_iterator()

    def get_permissions(self):
        '''
        Returns permissions for this directory or None if it's a special collection such as
        .session or .algo
        '''
        response = self.client.getHelper(self.url, acl='true')
        if response.status_code != 200:
            raise DataApiError('Unable to get permissions:' + str(response.content))
        content = response.json()
        if 'acl' in content:
            return Acl.from_acl_response(content['acl'])
        else:
            return None

    def update_permissions(self, acl):
        params = {'acl':acl.to_api_param()}
        response = self.client.patchHelper(self.url, params)
        if response.status_code != 200:
            raise DataApiError('Unable to update permissions: ' + response.json()['error']['message'])
        return True

    def _get_directory_iterator(self, type_filter=None):
        marker = None
        first = True
        while first or (marker is not None and len(marker) > 0):
            first = False
            url = self.url
            query_params= {}
            if marker:
                query_params['marker'] = marker
            response = self.client.getHelper(url, **query_params)
            if response.status_code != 200:
                raise DataApiError("Directory iteration failed: " + str(response.content))

            responseContent = response.content
            if isinstance(responseContent, six.binary_type):
                responseContent = responseContent.decode()

            content = json.loads(responseContent)
            if 'marker' in content:
                marker = content['marker']
            else:
                marker = None

            if type_filter is DataObjectType.directory or type_filter is None:
                for d in self._iterate_directories(content):
                    yield d
            if type_filter is DataObjectType.file or type_filter is None:
                for f in self._iterate_files(content):
                    yield f

    def _iterate_directories(self, content):
        directories = []
        if 'folders' in content:
            for dir_info in content['folders']:
                d = DataDirectory(self.client, pathJoin(self.path, dir_info['name']))
                d.set_attributes(dir_info)
                directories.append(d)
        return directories

    def _iterate_files(self, content):
        files = []
        if 'files' in content:
            for file_info in content['files']:
                f = DataFile(self.client, pathJoin(self.path, file_info['filename']))
                f.set_attributes(file_info)
                files.append(f)
        return files



'Algorithmia Algorithm API Client (python)'

import base64
import re
from Algmarket.async_response import AsyncResponse
from Algmarket.algm_response import AlgmResponse
from Algmarket.errors import ApiError, ApiInternalError
from enum import Enum

OutputType = Enum('OutputType','default raw void')

class Algorithm(object):
    def __init__(self, client, algoRef):
        # Parse algoRef
        algoRegex = re.compile(r"(?:algm://|/|)(\w+/.+)")
        m = algoRegex.match(algoRef)
        if m is not None:
            self.client = client
            self.path = m.group(1)
            self.url = '/v1/algm/' + self.path
            self.query_parameters = {}
            self.output_type = OutputType.default
        else:
            raise ValueError('Invalid algorithm URI: ' + algoRef)

    def set_options(self, timeout=300, stdout=False, output=OutputType.default, **query_parameters):
        self.query_parameters = {'timeout':timeout, 'stdout':stdout}
        self.output_type = output
        self.query_parameters.update(query_parameters)
        return self

    # Pipe an input into this algorithm
    def call(self, input1):

        if self.output_type == OutputType.raw:
            return self._postRawOutput(input1)
        elif self.output_type == OutputType.void:
            return self._postVoidOutput(input1)
        else:
            return AlgmResponse.create_algo_response(self.client.postJsonHelper(self.url, input1, **self.query_parameters))

    def _postRawOutput(self, input1):
            # Don't parse response as json
            self.query_parameters['output'] = 'raw'
            response = self.client.postJsonHelper(self.url, input1, parse_response_as_json=False, **self.query_parameters)
            # Check HTTP code and throw error as needed
            if response.status_code == 400:
                # Bad request
                raise ApiError(response.text)
            elif response.status_code == 500:
                raise ApiInternalError(response.text)
            else:
                return response.text

    def _postVoidOutput(self, input1):
            self.query_parameters['output'] = 'void'
            responseJson = self.client.postJsonHelper(self.url, input1, **self.query_parameters)
            if 'error' in responseJson:
                raise ApiError(responseJson['error']['message'])
            else:
                return AsyncResponse(responseJson)



'Algorithmia API Client (python)'

import Algmarket
from Algmarket.algorithm import Algorithm
from Algmarket.datafile import DataFile
from Algmarket.datadirectory import DataDirectory

import json, re, requests, six
import os

class Client(object):
    'Algorithmia Common Library'

    apiKey = None
    apiAddress = None

    def __init__(self, apiKey = None, apiAddress = None):
        # Override apiKey with environment variable
        if apiKey is None and 'ALGMARKET_KEY' in os.environ:
            apiKey = os.environ['ALGMARKET_KEY']
        self.apiKey = apiKey
        if apiAddress is not None:
            self.apiAddress = apiAddress
        else:
            self.apiAddress = Algmarket.getApiAddress()

    def algm(self, algmRef):
        return Algorithm(self, algmRef)

    def file(self, dataUrl):
        return DataFile(self, dataUrl)

    def dir(self, dataUrl):
        return DataDirectory(self, dataUrl)

    # Used internally to post json to the api and parse json response
    def postJsonHelper(self, url, input_object, parse_response_as_json=True, **query_parameters):
        headers = {}
        if self.apiKey is not None:
            headers['KeyValue'] = self.apiKey

        input_json = None
        if input_object is None:
            input_json = json.dumps(None).encode('utf-8')
            headers['Content-Type'] = 'application/json'
        elif isinstance(input_object, six.string_types):
            input_json = input_object.encode('utf-8')
            headers['Content-Type'] = 'text/plain'
        elif isinstance(input_object, bytearray) or isinstance(input_object, bytes):
            input_json = bytes(input_object)
            headers['Content-Type'] = 'application/octet-stream'
        else:
            input_json = json.dumps(input_object).encode('utf-8')
            headers['Content-Type'] = 'application/json'

        response = requests.post(self.apiAddress + url, data=input_json, headers=headers, params=query_parameters)

        if parse_response_as_json:
            return response.json()
        return response

    # Used internally to http get a file
    def getHelper(self, url, **query_parameters):
        headers = {}
        if self.apiKey is not None:
            headers['KeyValue'] = self.apiKey
        return requests.get(self.apiAddress + url, headers=headers, params=query_parameters)

    def patchHelper(self, url, params):
        headers = {'content-type': 'application/json'}
        if self.apiKey is not None:
            headers['KeyValue'] = self.apiKey
        return requests.patch(self.apiAddress + url, headers=headers, data=json.dumps(params))

    # Used internally to get http head result
    def headHelper(self, url):
        headers = {}
        if self.apiKey is not None:
            headers['KeyValue'] = self.apiKey
        return requests.head(self.apiAddress + url, headers=headers)

    # Used internally to http put a file
    def putHelper(self, url, data):
        headers = {}
        if self.apiKey is not None:
            headers['KeyValue'] = self.apiKey
        response = requests.put(self.apiAddress + url, data=data, headers=headers)
        return response.json()

    # Used internally to http delete a file
    def deleteHelper(self, url):
        headers = {}
        if self.apiKey is not None:
            headers['KeyValue'] = self.apiKey
        response = requests.delete(self.apiAddress + url, headers=headers)
        return response.json()



from enum import Enum

DataObjectType = Enum('DataObjectType','file directory')

class DataObject(object):
    def __init__(self, data_object_type):
        self.data_object_type = data_object_type

    def is_file(self):
        '''Returns whether object is a file'''
        return self.data_object_type is DataObjectType.file

    def is_dir(self):
        '''Returns whether object is a directory'''
        return self.data_object_type is DataObjectType.directory

    def get_type(self):
        '''Returns type of this DataObject'''
        return self.data_object_type

    def set_attributes(self):
        '''Sets attributes about the directory after querying the Data API'''
        raise NotImplementedError


'Algorithmia Data API Client (python)'

import re
import json
import six
import tempfile
from datetime import datetime

from Algmarket.util import getParentAndBase
from Algmarket.data import DataObject, DataObjectType
from Algmarket.errors import DataApiError

class DataFile(DataObject):
    def __init__(self, client, dataUrl):
        super(DataFile, self).__init__(DataObjectType.file)
        self.client = client
        # Parse dataUrl
        self.path = re.sub(r'^data://|^/', '', dataUrl)
        self.url = '/v1/data/' + self.path
        self.last_modified = None
        self.size = None

    def set_attributes(self, attributes):
        self.last_modified = datetime.strptime(attributes['last_modified'],'%Y-%m-%dT%H:%M:%S.000Z')
        self.size = attributes['size']

    # Deprecated:
    def get(self):
        return self.client.getHelper(self.url)

    # Get file from the data api
    def getFile(self):
        exists, error = self.existsWithError()
        if not exists:
            raise DataApiError('unable to get file {} - {}'.format(self.path, error))
        # Make HTTP get request
        response = self.client.getHelper(self.url)
        with tempfile.NamedTemporaryFile(delete = False) as f:
            for block in response.iter_content(1024):
                if not block:
                    break;
                f.write(block)
            f.flush()
            return open(f.name)

    def getName(self):
        _, name = getParentAndBase(self.path)
        return name

    def getBytes(self):
        exists, error = self.existsWithError()
        if not exists:
            raise DataApiError('unable to get file {} - {}'.format(self.path, error))
        # Make HTTP get request
        return self.client.getHelper(self.url).content

    def getString(self):
        exists, error = self.existsWithError()
        if not exists:
            raise DataApiError('unable to get file {} - {}'.format(self.path, error))
        # Make HTTP get request
        return self.client.getHelper(self.url).text

    def getJson(self):
        exists, error = self.existsWithError()
        if not exists:
            raise DataApiError('unable to get file {} - {}'.format(self.path, error))
        # Make HTTP get request
        return self.client.getHelper(self.url).json()

    def exists(self):
        # In order to not break backward compatability keeping this method to only return
        # a boolean
        exists, error = self.existsWithError()
        return exists

    def existsWithError(self):
        response = self.client.headHelper(self.url)
        error = None
        if 'X-Error-Message' in response.headers:
            error = response.headers['X-Error-Message']
        return (response.status_code == 200, error)

    def put(self, data):
        # Post to data api

        # First turn the data to bytes if we can
        if isinstance(data, six.string_types) and not isinstance(data, six.binary_type):
            data = bytes(data.encode())

        if isinstance(data, six.binary_type):
            result = self.client.putHelper(self.url, data)
            if 'error' in result:
                raise DataApiError(result['error']['message'])
            else:
                return self
        else:
            raise TypeError("Must put strings or binary data. Use putJson instead")

    def putJson(self, data):
        # Post to data api
        jsonElement = json.dumps(data)
        result = self.client.putHelper(self.url, jsonElement)
        if 'error' in result:
            raise DataApiError(result['error']['message'])
        else:
            return self

    def putFile(self, path):
        # Post file to data api
        with open(path, 'rb') as f:
            result = self.client.putHelper(self.url, f)
            if 'error' in result:
                raise DataApiError(result['error']['message'])
            else:
                return self

    def delete(self):
        # Delete from data api
        result = self.client.deleteHelper(self.url)
        if 'error' in result:
            raise DataApiError(result['error']['message'])
        else:
            return True



class Acl(object):
    def __init__(self, read_acl):
        self.read_acl = read_acl

    @staticmethod
    def from_acl_response(acl_response):
        '''Takes JSON response from API and converts to ACL object'''
        if 'read' in acl_response:
            read_acl = AclType.from_acl_response(acl_response['read'])
            return Acl(read_acl)
        else:
            raise ValueError('Response does not contain read ACL')

    def to_api_param(self):
        read_acl_string = self.read_acl.acl_string
        if read_acl_string is None:
            return {'read':[]}
        return {'read':[read_acl_string]}

class AclInner(object):
    def __init__(self, pseudonym, acl_string):
        self.pseudonym = pseudonym
        self.acl_string = acl_string

    def __repr__(self):
        return 'AclType(pseudonym=%s,acl_string=%s)' % (self.pseudonym, self.acl_string)

class AclType(object):
    public = AclInner('public','user://*')
    my_algos = AclInner('my_algos','algm://.my/*')
    private = AclInner('private',None)  # Really is an empty list
    default = my_algos

    types = (public, my_algos, private)

    @staticmethod
    def from_acl_response(acl_list):
        if len(acl_list) == 0:
            return AclType.private
        else:
            acl_string = acl_list[0]
            for t in AclType.types:
                if t.acl_string == acl_string:
                    return t
            else:
                raise ValueError('Invalid acl string %s' % (acl_list[0]))

class ReadAcl(object):
    public = Acl(AclType.public)
    private = Acl(AclType.private)
    my_algos = Acl(AclType.my_algos)



import os

from setuptools import setup

setup(
    name='algmarket',
    version='1.0.0',
    description='Algmarket Python Client',
    long_description='Algmarket Python Client is a client library for accessing Algorithmia from python code. This library also gets bundled with any Python algorithms in Algmarket.',
    url='https://github.com/qihe1126/algmarket-python',
    license='MIT',
    author='erniu',
    author_email='qihea163@163.com',
    packages=['Algmarket'],
    install_requires=[
        'requests',
        'six',
        'enum34'
    ],
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



import re

FNAME_MATCH = re.compile(r'/([^/]+)$')  # From the last slash to the end of the string 
PREFIX = re.compile(r'([^:]+://)(/)?(.+)')   # Check for a prefix like data://
def getParentAndBase(path):
    match = PREFIX.match(path)
    if match is None:
        if path.endswith('/'):
            stripped_path = path[:-1]
        else:
            stripped_path = path
        base = FNAME_MATCH.search(stripped_path)
        if base is None:
            raise ValueError('Invalid path')
        parent = FNAME_MATCH.sub('', stripped_path)
        return parent, base.group(1)
    else:
        prefix, leading_slash, uri = match.groups()
        parts = uri.split('/')
        parent_path = '/'.join(parts[:-1])

        if leading_slash is not None:
            parent_path = '{prefix}/{uri}'.format(prefix=prefix, uri='/'.join(parts[:-1]))
        else:
            parent_path = '{prefix}{uri}'.format(prefix=prefix, uri='/'.join(parts[:-1]))
        return parent_path, parts[-1]

def pathJoin(parent, base):
    if parent.endswith('/'):
        return parent + base
    return parent + '/' + base



'Algmarket API Client (python)'

from Algmarket.client import Client
import os

apiKey = None
apiAddress = None

# Get reference to an algorithm using a default client
def algm(algmRef):
    # Return algorithm reference using default client
    return getDefaultClient().algm(algmRef)

def file(dataUrl):
    return getDefaultClient().file(dataUrl)

def dir(dataUrl):
    return getDefaultClient().dir(dataUrl)

def client(api_key=None, api_address=None):
    return Client(api_key, api_address)

# The default client to use, assuming the user does not want to construct their own
defaultClient = None

# Used internally to get default client
def getDefaultClient():
    global defaultClient
    # Check for default client, and ensure default API key has not changed
    if defaultClient is None or defaultClient.apiKey is not apiKey:
        # Construct default client
        defaultClient = Client(apiKey)
    return defaultClient

# Used internally to get default api client
def getApiAddress():
    global apiAddress
    if apiAddress is not None:
        # First check for user setting Algorithmia.apiAddress = "XXX"
        return apiAddress
    elif 'ALGORITHMIA_API' in os.environ:
        # Then check for system environment variable
        return os.environ['ALGORITHMIA_API']
    else:
        # Else return default
        return "http://api.algmarket.com"



from enum import Enum

ModelObjectType = Enum('ModelObjectType','file directory')

class ModelObject(object):
    def __init__(self, model_object_type):
        self.model_object_type = model_object_type

    def is_file(self):
        '''Returns whether object is a file'''
        return self.model_object_type is ModelObjectType.file

    def is_dir(self):
        '''Returns whether object is a directory'''
        return self.model_object_type is ModelObjectType.directory

    def get_type(self):
        '''Returns type of this ModelObject'''
        return self.model_object_type

    def set_attributes(self):
        '''Sets attributes about the directory after querying the Model API'''
        raise NotImplementedError